/**
 * Pole Actor used by AdvKitPawn to climb on
 * 
 * 2013 by FreetimeCoder
 * www.freetimestudio.net
 */
class AdvKitPole extends Actor
	placeable
	ClassGroup(AdvKit);

/**collision cylinder, also used for offset calculation*/
var	() CylinderComponent    cylinderComponent;
/**mesh to show*/
var () StaticMeshComponent  mesh;


/**Returns the collision radius of our cylinder
 * collision component.
 *
 * @return	the collision radius of our pawn
 */
final simulated function float GetCollisionRadius()
{
	return cylinderComponent.CollisionRadius;
}

/**Returns the collision height of our cylinder
 * collision component.
 *
 * @return	collision height of our pawn
 */
final simulated function float GetCollisionHeight()
{
	return cylinderComponent.CollisionHeight;
}

/**@return a vector representing the box around this pawn's cylinder collision component, for use with traces */
final simulated final function vector GetCollisionExtent()
{
	local vector Extent;

	Extent = GetCollisionRadius() * vect(1,1,0);
	Extent.Z = GetCollisionHeight();
	return Extent;
}

/**Get the maximum height (top) of the pole, in case the cylinder component has an offset
 * 
 * @return  the maximum height
 */
final function float GetMaxHeight()
{
	return Location.Z + cylinderComponent.CollisionHeight + cylinderComponent.Translation.Z;
}

/**Get the minimum height (bottom) of the pole, in case the cylinder component has an offset
 * 
 * @return  the minimum height
 */
final function float GetMinHeight()
{
	return Location.Z - cylinderComponent.CollisionHeight + cylinderComponent.Translation.Z;
}

DefaultProperties
{
	begin object class=DynamicLightEnvironmentComponent name=LightEnv
	end object
	Components.Add(LightEnv);

	begin object class=StaticMeshComponent name=MeshComp0
		StaticMesh = StaticMesh'AdvKit_Assets.PoleClimbing.SM_Pole'
		LightEnvironment=LightEnv
	end object
	Components.Add(MeshComp0)
	mesh = MeshComp0

	Begin Object Class=CylinderComponent Name=CollisionCylinder
		CollisionRadius=10
		CollisionHeight=128
		BlockNonZeroExtent=true
		BlockZeroExtent=true
		BlockActors=true
		CollideActors=true
		HiddenEditor = false
	End Object
	CollisionComponent=CollisionCylinder
	cylinderComponent=CollisionCylinder
	Components.Add(CollisionCylinder)


	bCollideActors = true
	bBlockActors = true

	bEdShouldSnap = true

	RemoteRole=ROLE_SimulatedProxy
}
